/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "CommunicationTest.h"

using namespace advisor;

CommunicationTest::CommunicationTest( cube::CubeProxy* cube ) : PerformanceTest( cube )
{
    setName( tr( "Communication" ).toUtf8().data() );
    setWeight( 0.2 );   // need to be adjusted
    setValue( 0 );
    communication = cube->getMetric( "comm" );
    execution     = cube->getMetric( "execution" );

    if ( communication == NULL ||
         execution == NULL
         )
    {
        return;
    }

    cube::list_of_metrics      lmetrics;
    cube::list_of_sysresources lsysres = getRootsOfSystemTree();
    cube::metric_pair          metric;

    cube::value_container comm_inclusive_values;
    cube::value_container comm_exclusive_values;
    cube::value_container exec_inclusive_values;
    cube::value_container exec_exclusive_values;

    metric.first  = communication;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
    cube->getCallpathTreeValues( lmetrics,
                                 lsysres,
                                 comm_inclusive_values,
                                 comm_exclusive_values );
    lmetrics.clear();
    metric.first  = execution;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
    cube->getCallpathTreeValues( lmetrics,
                                 lsysres,
                                 exec_inclusive_values,
                                 exec_exclusive_values );
    for ( size_t i = 0; i < comm_inclusive_values.size(); ++i )
    {
        double comm_ivalue = ( comm_inclusive_values[ i ] )->getDouble();
        double exec_ivalue = ( exec_inclusive_values[ i ] )->getDouble();

        double comm_evalue = ( comm_exclusive_values[ i ] )->getDouble();
        double exec_evalue = ( exec_exclusive_values[ i ] )->getDouble();

        cube::Value* i_value = ( exec_inclusive_values[ i ] )->clone();
        i_value->
        operator=( 1 - ( comm_ivalue ) / exec_ivalue  );

        inclusive_values.push_back( i_value );
        cube::Value* e_value = ( exec_exclusive_values[ i ] )->clone();
        e_value->
        operator=( 1 - ( comm_evalue ) / exec_evalue  );

        exclusive_values.push_back( e_value );
    }
}



void
CommunicationTest::applyCnode( const cube::Cnode* cnode, const cube::CalculationFlavour cnf  )
{
    double comm_value = ( cnf == cube::CUBE_CALCULATE_INCLUSIVE ) ?
                        inclusive_values[ cnode->get_id() ]->getDouble() :
                        exclusive_values[ cnode->get_id() ]->getDouble();
    setValue( comm_value );
}

void
CommunicationTest::applyCnode( const cube::list_of_cnodes& cnodes )
{
    if ( cnodes.size() == 1 )
    {
        applyCnode( cnodes[ 0 ].first, cnodes[ 0 ].second );
    }
    else
    {
        cube::value_container comm_inclusive_values;
        cube::value_container comm_exclusive_values;
        cube::value_container exec_inclusive_values;
        cube::value_container exec_exclusive_values;

        cube::list_of_sysresources lsysres = getRootsOfSystemTree();
        cube::IdIndexMap           metric_id_indices;
        cube->getMetricSubtreeValues( cnodes,
                                      lsysres,
                                      *communication,
                                      1,
                                      metric_id_indices,
                                      &comm_inclusive_values,
                                      &comm_exclusive_values );

        cube->getMetricSubtreeValues( cnodes,
                                      lsysres,
                                      *execution,
                                      1,
                                      metric_id_indices,
                                      &exec_inclusive_values,
                                      &exec_exclusive_values );

        double comm_value = 1 - ( ( comm_inclusive_values[ 0 ]->getDouble() ) ) / ( exec_inclusive_values[ 0 ]->getDouble() );
        setValue( comm_value );
    }
}


const std::string&
CommunicationTest::getCommentText() const
{
    return no_comment;
}


// ------ overview tests ---------

bool
CommunicationTest::isActive() const
{
    return ( communication != NULL ) && ( execution != NULL );
};

bool
CommunicationTest::isIssue() const
{
    return false;
};
