/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "POPImbalanceTest.h"


using namespace advisor;

POPImbalanceTest::POPImbalanceTest( cube::CubeProxy* cube ) : PerformanceTest( cube )
{
    setName( tr( "    Load balance" ).toUtf8().data() );
    setWeight( 1 );   // need to be adjusted
    pop_imbalance = cube->getMetric( "lb_eff" );
    if ( pop_imbalance == NULL )
    {
        adjustForTest( cube );
    }
    pop_imbalance = cube->getMetric( "lb_eff" );
    if ( pop_imbalance == NULL )
    {
        setWeight( 0.1 );
        setValue( 0. );
        return;
    }
    cube::list_of_sysresources lsysres = getRootsOfSystemTree();
    cube::metric_pair          metric;
    metric.first  = pop_imbalance;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
}



void
POPImbalanceTest::applyCnode( const cube::Cnode* cnode, const cube::CalculationFlavour cnf  )
{
    if ( pop_imbalance == NULL )
    {
        return;
    }
    cube::value_container      inclusive_values;
    cube::value_container      exclusive_values;
    cube::list_of_sysresources lsysres;
    cube::IdIndexMap           metric_id_indices;
    cube->getCallpathSubtreeValues( lmetrics,
                                    lsysres,
                                    *( const_cast<cube::Cnode*>( cnode ) ),
                                    0,
                                    metric_id_indices,
                                    &inclusive_values,
                                    &exclusive_values );
    double pop_imbalance_value = ( cnf == cube::CUBE_CALCULATE_INCLUSIVE ) ?
                                 inclusive_values[ 0 ]->getDouble() :
                                 exclusive_values[ 0 ]->getDouble();
    setValue( pop_imbalance_value );
}

void
POPImbalanceTest::applyCnode( const cube::list_of_cnodes& cnodes )
{
    if ( pop_imbalance == NULL )
    {
        return;
    }
    cube::value_container      inclusive_values;
    cube::value_container      exclusive_values;
    cube::list_of_sysresources lsysres;
    cube::IdIndexMap           metric_id_indices;
    cube->getMetricSubtreeValues( cnodes,
                                  lsysres,
                                  *pop_imbalance,
                                  0,
                                  metric_id_indices,
                                  &inclusive_values,
                                  &exclusive_values );
    double pop_imbalance_value = inclusive_values[ 0 ]->getDouble();
    setValue( pop_imbalance_value );
}


const std::string&
POPImbalanceTest::getCommentText() const
{
    if ( pop_imbalance == NULL )
    {
        return no_comment;
    }
    else
    {
        return no_comment;
    }
}


// ------ overview tests ---------

bool
POPImbalanceTest::isActive() const
{
    return pop_imbalance != NULL;
};

bool
POPImbalanceTest::isIssue() const
{
    return false;
};

void
POPImbalanceTest::adjustForTest( cube::CubeProxy* cube ) const
{
    cube::Metric* max_comp_time = cube->getMetric( "max_comp_time" );
    if ( max_comp_time == NULL )
    {
        add_max_comp_time( cube );
    }
    cube::Metric* avg_comp_time = cube->getMetric( "avg_comp_time" );
    if ( avg_comp_time == NULL )
    {
        add_avg_comp_time( cube );
    }
    max_comp_time = cube->getMetric( "max_comp_time" );
    avg_comp_time = cube->getMetric( "avg_comp_time" );
    if ( max_comp_time != NULL && avg_comp_time != NULL )
    {
        add_lb_eff( cube );
    }
}


void
POPImbalanceTest::add_lb_eff( cube::CubeProxy* ) const
{
    cube::Metric* _met = cube->defineMetric(
        tr( "Load balance efficiency" ).toUtf8().data(),
        "lb_eff",
        "DOUBLE",
        "",
        "",
        POP_LB_EFF_METRIC_URL,
        tr( "Load balance efficiency reflects how well the distribution of work to processes of threads is done in the application. It is computed as a ratio between the average time of a process spend in computation and the maximum time a process spends in computation, i.e. ( avg_comp_time / max_comp_time ). The value varies from 0 to 100%, where 0 shows that it is a MPI routine. Do not consider values in the flat view and the system tree." ).toUtf8().data(),
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "metric::avg_comp_time() / metric::max_comp_time()",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    if ( _met != NULL )
    {
        _met->setConvertible( false );
    }
    advisor_services->addMetric( _met, NULL );
}

void
POPImbalanceTest::add_avg_comp_time( cube::CubeProxy* ) const
{
    cube::Metric* _comp = cube->getMetric( "comp" );
    if ( _comp == NULL )
    {
        add_comp_time( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        tr( "AVG computation time" ).toUtf8().data(),
        "avg_comp_time",
        "DOUBLE",
        tr( "sec" ).toUtf8().data(),
        "",
        POP_LB_EFF_METRIC_URL,
        tr( "AVG computation time, ( comp / number_of_processes )" ).toUtf8().data(),
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "metric::comp() / ${cube::#locations::nonvoid}",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    if ( _met != NULL )
    {
        _met->setConvertible( false );
    }
    advisor_services->addMetric( _met, NULL );
}

void
POPImbalanceTest::add_max_comp_time( cube::CubeProxy* ) const
{
    cube::Metric* _comp = cube->getMetric( "comp" );
    if ( _comp == NULL )
    {
        add_comp_time( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        tr( "Maximal computation time" ).toUtf8().data(),
        "max_comp_time",
        "DOUBLE",
        tr( "sec" ).toUtf8().data(),
        "",
        POP_LB_EFF_METRIC_URL,
        tr( "Maximal computation time, max( comp )" ).toUtf8().data(),
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "metric::comp()",
        "",
        "",
        "",
        "max(arg1, arg2)",
        true,
        cube::CUBE_METRIC_GHOST
        );
    if ( _met != NULL )
    {
        _met->setConvertible( false );
    }
    advisor_services->addMetric( _met, NULL );
}
