/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "POPSerialisationTest.h"


using namespace advisor;

POPSerialisationTest::POPSerialisationTest( cube::CubeProxy* cube ) : PerformanceTest( cube )
{
    setName( tr( "         Serialisation Eficiency" ).toUtf8().data() );
    setWeight( 1 );   // need to be adjusted
    pop_serialisation = cube->getMetric( "ser_eff" );
    if ( pop_serialisation == NULL )
    {
        adjustForTest( cube );
    }
    pop_serialisation = cube->getMetric( "ser_eff" );
    if ( pop_serialisation == NULL )
    {
        setWeight( 0.1 );
        setValue( 0. );
        return;
    }
    cube::list_of_sysresources lsysres = getRootsOfSystemTree();
    cube::metric_pair          metric;
    metric.first  = pop_serialisation;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
}



void
POPSerialisationTest::applyCnode( const cube::Cnode* cnode, const cube::CalculationFlavour cnf  )
{
    if ( pop_serialisation == NULL )
    {
        return;
    }
    cube::value_container      inclusive_values;
    cube::value_container      exclusive_values;
    cube::list_of_sysresources lsysres;
    cube::IdIndexMap           metric_id_indices;
    cube->getCallpathSubtreeValues( lmetrics,
                                    lsysres,
                                    *( const_cast<cube::Cnode*>( cnode ) ),
                                    0,
                                    metric_id_indices,
                                    &inclusive_values,
                                    &exclusive_values );
    double pop_serialisation_value = ( cnf == cube::CUBE_CALCULATE_INCLUSIVE ) ?
                                     inclusive_values[ 0 ]->getDouble() :
                                     exclusive_values[ 0 ]->getDouble();
    setValue( pop_serialisation_value );
}

void
POPSerialisationTest::applyCnode( const cube::list_of_cnodes& cnodes )
{
    if ( pop_serialisation == NULL )
    {
        return;
    }
    cube::value_container      inclusive_values;
    cube::value_container      exclusive_values;
    cube::list_of_sysresources lsysres;     // = getRootsOfSystemTree();
    cube::IdIndexMap           metric_id_indices;
    cube->getMetricSubtreeValues( cnodes,
                                  lsysres,
                                  *pop_serialisation,
                                  0,
                                  metric_id_indices,
                                  &inclusive_values,
                                  &exclusive_values );
    double pop_serialisation_value = inclusive_values[ 0 ]->getDouble();
    setValue( pop_serialisation_value );
}


const std::string&
POPSerialisationTest::getCommentText() const
{
    if ( pop_serialisation == NULL )
    {
        return no_comment;
    }
    else
    {
        return no_comment;
    }
}


// ------ overview tests ---------

bool
POPSerialisationTest::isActive() const
{
    return pop_serialisation != NULL;
};

bool
POPSerialisationTest::isIssue() const
{
    return false;
};

void
POPSerialisationTest::adjustForTest( cube::CubeProxy* cube ) const
{
    if ( scout_metrics_available( cube ) )
    {
        add_ser_eff( cube );
    }
}


void
POPSerialisationTest::add_ser_eff( cube::CubeProxy* ) const
{
    cube::Metric* _comp = cube->getMetric( "comp" );
    if ( _comp == NULL )
    {
        add_comp_time( cube );
    }
    cube::Metric* _transfer_time_mpi = cube->getMetric( "transfer_time_mpi" );
    if ( _transfer_time_mpi == NULL )
    {
        add_transfer_time_mpi( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        tr( "Serialization efficiency per process" ).toUtf8().data(),
        "ser_eff",
        "DOUBLE",
        "%",
        "",
        POP_SER_EFF_METRIC_URL,
        tr( "Serialization efficiency reflects the loss of efficiency due to dependencies among processes. It is computed as a maximum ratio of a process in computation time on ideal network to total runtime on ideal network, i.e. ( comp / ( execution - transfer_time_mpi ) ). The value varies from 0 to 100%, where 100% shows that routine is a pure computation routine, whereas 0 shows that it is a MPI routine. Do not consider values in the flat view and the system tree." ).toUtf8().data(),
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "{"
        " if( metric::fixed::mpi_indicator()>0)"
        "{"
        "    ${tmp}= metric::fixed::mpi_indicator(i) * metric::comp() /  ( metric::execution() - metric::transfer_time_mpi() );"
        "}"
        "else"
        "{"
        "    ${tmp} = metric::comp() / ( metric::execution() - metric::transfer_time_mpi() );"
        "};"
        "   return ${tmp};"
        "}",
        "",
        "",
        "",
        "max(arg1, arg2)",
        true,
        cube::CUBE_METRIC_GHOST
        );
    if ( _met != NULL )
    {
        _met->setConvertible( false );
    }
    advisor_services->addMetric( _met, NULL );
}

void
POPSerialisationTest::add_transfer_time_mpi( cube::CubeProxy* ) const
{
    cube::Metric* _comp = cube->getMetric( "mpi" );
    if ( _comp == NULL )
    {
        add_mpi_time( cube );
    }
    cube::Metric* _mpi_io = cube->getMetric( "mpi_io" );
    if ( _mpi_io == NULL )
    {
        add_mpi_io_time( cube );
    }
    cube::Metric* _wait_time_mpi = cube->getMetric( "wait_time_mpi" );
    if ( _wait_time_mpi == NULL )
    {
        add_wait_time_mpi( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        tr( "Transfer time in MPI" ).toUtf8().data(),
        "transfer_time_mpi",
        "DOUBLE",
        tr( "sec" ).toUtf8().data(),
        "",
        POP_SER_EFF_METRIC_URL,
        tr( "Transfer time in MPI, (mpi - wait_time_mpi - mpi_io)" ).toUtf8().data(),
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "metric::mpi() - metric::wait_time_mpi() - metric::mpi_io()",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    if ( _met != NULL )
    {
        _met->setConvertible( false );
    }
    advisor_services->addMetric( _met, NULL );
}
